#ifndef XMEMORY_H
#define XMEMORY_H
/****************************************
*  Computer Algebra System SINGULAR     *
****************************************/
/* $Id: mmemory.h,v 1.9 1998/06/02 15:29:58 Singular Exp $ */
/*
* ABSTRACT
*/
#include <stdlib.h>
#include <string.h>

#ifdef __cplusplus
extern "C" {
#endif

struct omBin_s
{
  size_t s;
};  
typedef struct omBin_s      omBin_t;
typedef omBin_t*            omBin;

struct omInfo_s;
typedef struct omInfo_s omInfo_t;
struct omInfo_s
{
  long MaxBytesSystem;      /* set in omUpdateInfo(), is more accurate with malloc support   */
  long CurrentBytesSystem;  /* set in omUpdateInfo(), is more accurate with malloc support */
  long MaxBytesSbrk;        /* always up-to-date, not very accurate, needs omInintInfo() */
  long CurrentBytesSbrk;    /* set in omUpdateInfo(), needs omInintInfo() */
  long MaxBytesMmap;        /* set in omUpdateInfo(), not very accurate */
  long CurrentBytesMmap;    /* set in omUpdateInfo(), not very accurate */
  long UsedBytes;           /* set in omUpdateInfo() */
  long AvailBytes;          /* set in omUpdateInfo() */
  long UsedBytesMalloc;     /* set in omUpdateInfo(), needs malloc support */
  long AvailBytesMalloc;    /* set in omUpdateInfo(), needs malloc support */
  long MaxBytesFromMalloc;      /* always kept up-to-date */
  long CurrentBytesFromMalloc;  /* always kept up-to-date */
  long MaxBytesFromValloc;      /* always kept up-to-date */
  long CurrentBytesFromValloc;  /* always kept up-to-date */
  long UsedBytesFromValloc; /* set in omUpdateInfo()  */
  long AvailBytesFromValloc;/* set in omUpdateInfo()  */
  long MaxPages;            /* always kept up-to-date */
  long UsedPages;           /* always kept up-to-date */
  long AvailPages;          /* always kept up-to-date */
  long MaxRegionsAlloc;     /* always kept up-to-date */
  long CurrentRegionsAlloc; /* always kept up-to-date */
};

extern struct omInfo_s om_Info;

struct omOpts_s;
extern struct omOpts_s
{
  int MinTrack;
  int MinCheck;
  int MaxTrack;
  int MaxCheck;
  int Keep;
  int HowToReportErrors;
  int MarkAsStatic;
  unsigned int PagesPerRegion;
  void (*OutOfMemoryFunc)();
  void (*MemoryLowFunc)();
  void (*ErrorHook)();
} om_Opts;

typedef struct omOpts_s omOpts_t;

extern int om_sing_opt_show_mem;

#define omSizeWOfAddr(P)         ((mmSizeL(P)-sizeof(void*))/sizeof(long))
#define omSizeOfAddr(P)          (mmSizeL(P)-sizeof(void*))
#define omTypeAllocBin(T,P,B)    P=(T)AllocL((B)->s)
#define omTypeAlloc(T,P,S)       P=(T)AllocL(S)
#define omTypeAlloc0Bin(T,P,B)   P=(T)AllocL0((B)->s)
#define omAlloc(S)               AllocL(S)
#define omAlloc0(S)              AllocL0(S)
#define omAlloc0Aligned(S)       AllocL0(S)
#define omAllocAligned(S)        AllocL(S)
#define omAllocBin(B)            AllocL((B)->s)
#define omAlloc0Bin(B)           AllocL0((B)->s)
#define omInitInfo()             
#define omInitGetBackTrace()
#define omUpdateInfo()             
#define omPrintStats(F)
#define omPrintInfo(F)
#define omPrintBinStats(F)
#define omMarkMemoryAsStatic()
#define omfree(P)                FreeL(P)
#define omFree(P)                FreeL(P)
#define omFreeBin(P,B)           FreeL(P)
#define omFreeSize(P,S)          FreeL(P)
#define omfreeSize(P,S)          FreeL(P)
#define omFreeBinAddr(P)         FreeL(P)
#define omRealloc(A,NS)          mmRealloc(A,NS)
#define omReallocSize(A,OS,NS)   mmRealloc(A,NS)
#define omreallocSize(A,OS,NS)   mmRealloc(A,NS)
#define omRealloc0Size(A,OS,NS)  mmRealloc0(A,NS)
#define omrealloc0Size(A,OS,NS)  mmRealloc0(A,NS)
#define omMarkAsStaticAddr(A)
#define omMemCpyW(A,B,S)         memcpy(A,B,(S)<<2);
#define omMemcpyW(A,B,S)         memcpy(A,B,(S)<<2);


/* debug dummies: */
#define omTypeReallocAlignedSize     omTypeReallocSize
#define omTypeRealloc0AlignedSize    omTypeRealloc0Size
#define omReallocAlignedSize         omReallocSize
#define omRealloc0AlignedSize        omRealloc0Size
#define omMemDupAligned     omMemDup
#define omCheckIf(cond, test)                    ((void) 0)
#define omCheckBinAddr(addr)                     ((void) 0)
#define omCheckAddrBin(addr,bin)                 ((void) 0)
#define omCheckBinAddrSize(addr,size)            ((void) 0)
#define omCheckAddrSize(addr,size)               ((void) 0)
#define omCheckAddr(addr)                        ((void) 0)
#define omcheckAddrSize(addr,size)               ((void) 0)
#define omcheckAddr(addr)                        ((void) 0)
#define omCheckBin(bin)                          ((void) 0)
#define omCheckMemory()                          ((void) 0)


omBin  omGetSpecBin(size_t);   
#define omGetStickyBinOfBin(B) omGetSpecBin((B)->s)
void omMergeStickyBinIntoBin(omBin sticky_bin, omBin into_bin);
void omUnGetSpecBin(omBin *bin_p);

void omInitRet_2_Info(const char* argv0);
char * omFindExec (const char *name, char* executable);

void * omMemDup(void *);

void mmSpecializeBlock( size_t );

int mmMemReal( void );
int mmMemUsed( void );
#ifdef HAVE_SBRK
int mmMemPhysical( void );
#endif

#define omStrDup mmStrdup
/* char * mstrdup(char *s); */

void memcpyW(void* p1, void* p2, long l);

size_t mmSizeL( void* );
#define mmGetSpecSize() mm_specSize

extern size_t mm_specSize;

void * mmAllocBlock( size_t );
void * mmAllocBlock0( size_t );

void mmFreeBlock( void*, size_t );

void * mmReallocBlock( void*, size_t, size_t );

void * mmAllocSpecialized( void );

void mmFreeSpecialized( void* );

void * mmAlloc( size_t );
void * mmAlloc0( size_t );

void * mmRealloc( void*, size_t );
void * mmRealloc0( void*, size_t );
void * mmReallocSize( void*, size_t, size_t );

void mmFreeSize(void* adr, size_t);

void mmFree( void* );

char * mmStrdup( const char* );

#define Free mmFreeBlock
#define Alloc mmAllocBlock
#define Alloc0 mmAllocBlock0
#define ReAlloc mmReallocBlock
#define FreeL mmFree
#define AllocL mmAlloc
#define AllocL0 mmAlloc0

#ifdef __cplusplus
}
#endif

#undef OMALLOC_USES_MALLOC
#define X_OMALLOC
#define omMallocFunc mmAlloc
#define omReallocSizeFunc mmReallocSize
#define omFreeSizeFunc mmFreeSize
/* #define OM_NDEBUG */
#undef OM_SING_KEEP
#endif
